"""Statusline — plugin dispatcher and session utilities for Claude Code."""

from .discovery import discover_chatmap, find_session_metadata
from .paths import PLUGIN_DIR, SESSION_BASE, plugin_name, session_dir
from .plugin import ensure_plugin, install_builtins, list_plugins

__all__ = [
    "PLUGIN_DIR",
    "SESSION_BASE",
    "discover_chatmap",
    "ensure_plugin",
    "find_session_metadata",
    "install_builtins",
    "list_plugins",
    "plugin_name",
    "session_dir",
]
